# SPDX-FileCopyrightText: 2024 Brandenburgische Technische Universität Cottbus-Senftenberg
#
# SPDX-License-Identifier: EUPL-1.2

"""
PET plugin API wrapper.
"""

######################################################
# import commonly used libraries. You can add/remove
# libraries as needed
######################################################

import os
import json
from pathlib import Path
import subprocess

# from evaluation_system.api.exceptions import PluginError
from typing import List, Optional

from evaluation_system.api import plugin
from evaluation_system.api.parameters import (
    Directory,
    Integer,
    ParameterDictionary,
    File,
    SolrField,
    String,
    Bool,
    Date
    )
from evaluation_system.model.user import User
from evaluation_system.misc import logger

import freva

class PET(plugin.PluginAbstract):
    ######################################################
    # Define the details for the plugin
    ######################################################

    __version__ = (2025, 1, 10)
    __short_description__: str = (
        "Calculation of daily Potential Evatransporation"
    )
    __category__: Optional[str] = "postproc"
    __tags__: Optional[List[str]] = ["Potential Evatranspiration"]
    tool_developer = {
        "name": "Christian Beier",
        "email": "christian.beier@b-tu.de",
    }
    __long_description__: Optional[
        str
    ] = "This plugin calculates daily Potential Evapotranspiration (PET) based on the methodology outlined in FAO Irrigation and Drainage Paper No. 56 (Crop Evapotranspiration) by Allen, Pereira et al., 1998.\n\nFor more details, refer to the official publication: https://www.fao.org/4/x0490e/x0490e00.htm"

    ######################################################
    # Define the parameters for the plugin * Mandatory
    ######################################################

    __parameters__ = ParameterDictionary(
        SolrField(
            name="project",
            facet="project",
            mandatory=True,
            help="Project facet for input data search.",
            predefined_facets={
                "time_frequency": ["1day"],
                "variable": ["tasmax", "tasmin", "sfcWind", "pvap", "rsds", "orog"],
            },
        ),
        SolrField(
            name="product",
            facet="product",
            mandatory=True,
            help="Product facet for input data search.",
            predefined_facets={
                "time_frequency": ["1day"],
                "variable": ["tasmax", "tasmin", "sfcWind", "pvap", "rsds", "orog"],
            }),
        SolrField(
            name="institute",
            facet="institute",
            mandatory=True,
            help="Institute facet for input data search.",
            predefined_facets={
                "time_frequency": ["1day"],
                "variable": ["tasmax", "tasmin", "sfcWind", "pvap", "rsds", "orog"],
            },
        ),
        SolrField(
            name="model",
            facet="model",
            mandatory=True,
            help="Model facet for input data search.",
            predefined_facets={
                "time_frequency": ["1day"],
                "variable": ["tasmax", "tasmin", "sfcWind", "pvap", "rsds", "orog"],
            },
        ),
        SolrField(
            name="experiment",
            facet="experiment",
            mandatory=True,
            help="Experiment facet for input data search.",
            predefined_facets={
                "time_frequency": ["1day"],
                "variable": ["tasmax", "tasmin", "sfcWind", "pvap", "rsds", "orog"],
            },
        ),
        SolrField(
            name="realm",
            facet="realm",
            mandatory=True,
            help="Realm facet for input data search.",
            predefined_facets={
                "time_frequency": ["1day"],
                "variable": ["tasmax", "tasmin", "sfcWind", "pvap", "rsds", "orog"],
            },
        ),
        SolrField(
            name="ensemble",
            facet="ensemble",
            mandatory=True,
            help="ensemble facet for input data search.",
            predefined_facets={
                "time_frequency": ["1day"],
                "variable": ["tasmax", "tasmin", "sfcWind", "pvap", "rsds", "orog"],
            },
        ),
        Date(
            name="start_date",
            mandatory=True,
            help="Insert start date you want to select (YYYY-MM-DD)"
        ),
        Date(
            name="end_date",
            mandatory=True,
            help="Insert end date you want to select (YYYY-MM-DD)"
            ),
        String(
            name="region",
            default=None,
            help="Region box to be processed. The format have to be W,E,S,N (e.g. -180,180,0,90 for NH).\nLeave empty to select the region from input grid."
        ),
        Directory(
            name="output_dir",
            default="$USER_OUTPUT_DIR",
            help="Directory where output will be stored.",
        ),
        Bool(
            name="debug",
            default=False,
            help="Turn on to get better detailed debug information in case you face any issue."
        )
    )


    def find_files(self, config_dict, variables: List[str], time_frequency: str, time: Optional[str] = None):
        '''
        Selects data files based on the given parameters using the freva databrowser

            Parameters:
                config_dict (dict): Dictionary containing values used for searching the database
                variables (list): List of variables used to search data files within the database
                time_frequency (str): Abbreviation for time frequency the data should cover
                time (str): Time period the data should cover
            Returns:
                file_dict (dict): Dictionary containing list of files for each variable
        '''
        
        search_keys = (
            "project",
            "product",
            "institute",
            "model",
            "experiment",
            "realm",
            "ensemble"
        )

        search_args = {key: config_dict[key] for key in search_keys}
        search_args["time_frequency"] = time_frequency
        search_args["time"] = time

        file_dict = {}
        for variable in variables:
            search_args["variable"] = variable
            if variable == "orog":
                del search_args["ensemble"] ## orog files are part of another ensemble than the other variables
            files = list(freva.databrowser(**search_args))
            file_dict[variable] = files
        
        return file_dict

    def prepare_html_output(self, path_output):
        """
        Prepare the HTML output for the plugin in 
        the result section.
        """
        result = subprocess.run(
            ["ls", "-la", path_output], capture_output=True, text=True
        )
        ls_output = result.stdout
        current_user = User()

        html_content = f"""<!DOCTYPE html>
        <html>
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0">
            <style>
                body {{
                    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, sans-serif;
                    margin: 0;
                    padding: 20px;
                    line-height: 1.6;
                    background-color: white;
                    display: flex;
                    flex-direction: column;
                    align-items: center;
                    min-height: 100vh;
                }}
                .info-text {{
                    font-size: 16px;
                    max-width: 800px;
                    margin: 0 auto 20px auto;
                    color: #0c5460;
                    background-color: #d1ecf1;
                    border: 1px solid #bee5eb;
                    padding: 20px;
                    border-radius: 8px;
                    width: 100%;
                    box-sizing: border-box;
                }}
                .terminal {{
                    background-color: #2d2d2d;
                    border-radius: 8px;
                    box-shadow: 0 10px 20px rgba(0,0,0,0.3);
                    width: 100%;
                    max-width: 800px;
                    overflow: hidden;
                }}
                .terminal-header {{
                    background-color: #3d3d3d;
                    padding: 10px;
                    display: flex;
                    align-items: center;
                }}
                .terminal-buttons {{
                    display: flex;
                    gap: 8px;
                }}
                .terminal-button {{
                    width: 12px;
                    height: 12px;
                    border-radius: 50%;
                }}
                .terminal-button.close {{background-color: #ff5f56;}}
                .terminal-button.minimize {{background-color: #ffbd2e;}}
                .terminal-button.maximize {{background-color: #27c93f;}}
                .terminal-title {{
                    color: #ababab;
                    font-size: 14px;
                    margin-left: 20px;
                }}
                .terminal-body {{
                    padding: 20px;
                    font-family: "Menlo", "Monaco", "Consolas", monospace;
                    font-size: 14px;
                    line-height: 1.5;
                    color: #fff;
                    overflow-x: auto;
                    -webkit-overflow-scrolling: touch;
                }}
                .prompt {{
                    color: #98c379;
                    margin-bottom: 10px;
                    word-break: break-all;
                }}
                .command {{
                    color: #61afef;
                    margin-left: 8px;
                }}
                .output {{
                    color: #abb2bf;
                    white-space: pre-wrap;
                    margin: 10px 0 20px 0;
                }}
                @media (max-width: 480px) {{
                    body {{ padding: 10px; }}
                    .terminal-body {{ padding: 15px; }}
                }}                
            </style>
        </head>
        <body>
            <div class="info-text">
                You can directly download the produced data from the link above or login to levante and and proceed with the exploration of the data.
            </div>
            <div class="terminal">
                <div class="terminal-header">
                    <div class="terminal-buttons">
                        <div class="terminal-button close"></div>
                        <div class="terminal-button minimize"></div>
                        <div class="terminal-button maximize"></div>
                    </div>
                    <span class="terminal-title">Terminal</span>
                </div>
                <div class="terminal-body">
                    <div class="prompt">
                        local:~$ <span class="command">ssh {current_user.getName()}@levante.dkrz.de</span>
                    </div>
                    <div class="prompt">
                        {current_user.getName()}@levante.dkrz.de:~$ <span class="command">cd {path_output}</span>
                    </div>
                    <div class="prompt">
                        {current_user.getName()}@levante.dkrz.de:{path_output}$ <span class="command">ls -la {path_output}</span>
                    </div>
                    <div class="output">{ls_output}</div>
                </div>
            </div>
        </body>
        </html>"""

        html_output_path = os.path.join(path_output, "02_view.html")
        with open(html_output_path, "w") as f:
            f.write(html_content)


    def run_tool(self, config_dict=None):

        config_dict["files"] = self.find_files(config_dict, ["tasmax", "tasmin", "sfcWind", "pvap", "rsds"], "day", config_dict["start_date"] + " to " + config_dict["end_date"])
        config_dict["alti_file"] = self.find_files(config_dict, ["orog"], "fx")["orog"]
        if "alti_file" not in config_dict or not config_dict["alti_file"]:
            logger.error("Can't find altitude file")
        
        out_dir = self._special_variables.substitute({"d": "$USER_OUTPUT_DIR"})
        out_dir = Path(out_dir["d"]) / str(self.rowid)
        out_dir.mkdir(exist_ok=True, parents=True)
        config_dict["output_dir"] = str(out_dir)

        user = User()
        scratch = os.getenv("SCRATCH", f"/scratch/{user.getName()[0]}/{user.getName()}")
        cache_dir = Path(f"{scratch}/pet/{self.rowid}")
        cache_dir.mkdir(exist_ok=True, parents=True)
        config_dict["cache_dir"] = str(cache_dir)

        json_file = Path(config_dict["cache_dir"]) / "out.json"
        error_file = Path(config_dict["cache_dir"]) / "error.log"

        with json_file.open("w") as json_f:
            json.dump(config_dict, json_f, indent=4)

        tool_path = Path(__file__).parent / "src" / "pet" / "calculate_pet.sh"

        with error_file.open("w") as error_f:
            self.call(
                f"{tool_path} {json_file}",
                stderr=error_f,
            )
        self.prepare_html_output(config_dict["output_dir"])

        return self.prepare_output(config_dict["output_dir"])
